function [len, Z, rocfun, exitflag, critical, fx, fy, transx, transy]=roclength(x,y,method)

%INPUT ARGUMENTS:
%x: biomarker scores for the controls
%y: biomarker scores for the cases
%method: It can be one of the following 
%1:'boxcox' : applies the Box-Cox approach and considers the asymptotic inference
%2:'normal' : assumes normality (bi-normal) and considers the asymptotic inference
%3:'kernelr': kernel based length and Z is based on 1000 permutations (permutation test)
%4:'boxcoxr': Box-Cox based length and Z is based on 1000 permutations (permutation test)
%5:'normalr': normality assymption (bi-normal) and Z is based on 1000 permutations (permutation test)

%OUTPUT ARGUMENTS:
%len:      estimated length
%Z:        value of the proposed statistic (theorem 3 of Bantis et al. paper)
%rocfun:   a function handle of the estimated ROC curve
%critical: critical value based on which statistical significance is going to assessed
%fx:       a function handle for the density of the controls
%fy:       a function handle for the density of the cases (diseased group)
%transx:   the transformed data of the controls (if a transformation is applied). This output argument refers only to the Box-Cox method. Do not request it is the Box-Cox is not the method of interest).
%transy:   the transformed data of the cases (if a transformation is applied). This output argument refers only to the Box-Cox method. Do not request it is the Box-Cox is not the method of interest).

transx=[];
transy=[];


switch method

    
    case 'boxcox'  
lastwarn('')
[transx, transy, lam, exitflag]=boxcoxleo(x,y);
x=transx;
y=transy;
[mx sx]=normfit(x);
[my sy]=normfit(y);
fx=@(x) normpdf(x,mx,sx);
fy=@(y) normpdf(y,my,sy);

l1=quadgk(@(x) sqrt(fx(x).^2+fy(x).^2),-Inf,Inf);

%=============START CALCULATING THE LENGTH===========================

Z=4*sqrt(2)*length(x)*length(y)/(length(x)+length(y)).*(l1-sqrt(2));
len=l1;
rocfun=@(t) 1-normcdf(norminv(1-t,mx,sx),my,sy);
if isempty(lastwarn)==1
    exitflag=1;
else
    exitflag=0;
end


critical=    chi2inv(1-0.05,2);


    case 'normal'  
lastwarn('')

[mx sx]=normfit(x);
[my sy]=normfit(y);
fx=@(x) normpdf(x,mx,sx);
fy=@(y) normpdf(y,my,sy);

l1=quadgk(@(x) sqrt(fx(x).^2+fy(x).^2),0,50);

%=============START CALCULATING THE LENGTH===========================

Z=4*sqrt(2)*length(x)*length(y)/(length(x)+length(y)).*(l1-sqrt(2));
len=l1;
rocfun=@(t) 1-normcdf(norminv(1-t,mx,sx),my,sy);
if isempty(lastwarn)==1
    exitflag=1;
else
    exitflag=0;
end


critical=    chi2inv(1-0.05,2);



    case 'kernel'  
lastwarn('')

xor=x;yor=y;nx=length(x);ny=length(y);
    h1=0.9*min(std(xor),iqr(xor)/1.34)*length(xor)^(-0.2);
    h2=0.9*min(std(yor),iqr(yor)/1.34)*length(yor)^(-0.2);
    
    fx =@(at) ksdensity(xor,at,'width',h1,'function','pdf');
    fy =@(at) ksdensity(yor,at,'width',h2,'function','pdf');
    len=quadgk(@(x) sqrt(fx(x).^2+fy(x).^2),-Inf,Inf);
    F1inv =@(at) ksdensity(xor,at,'width',h1,'function','icdf');
    F2 =@(at) ksdensity(yor,at,'width',h2,'function','cdf');
    rocfun=@(t) 1-F2(F1inv(1-t));
    
    Z=4*sqrt(2)*length(x)*length(y)/(length(x)+length(y)).*(len-sqrt(2));

        
  boots=1; 
  pooled=[xor;yor];
%=============START CALCULATING THE LENGTH===========================
while boots<=1000
        rand('twister', 5493+boots);
      randn('state', 3+boots);
      randn('seed',3+boots);
      rand('seed',3+boots);

    warn=0;
    boots;
    if boots==200 || boots==400 || boots==600 || boots==800
        boots
    end
    
 %ata=randsample(1:nx,nx,'true');
    %atb=randsample(1:ny,ny,'true');
    
    ata=randperm(nx+ny);
    
    pooledboots=pooled(ata);
    xboots=pooledboots(1:nx);
    yboots=pooledboots((nx+1):end);
    
      
    %===KERNEL BASED==========================
    h1=0.9*min(std(xboots),iqr(xboots)/1.34)*length(xboots)^(-0.2);
    h2=0.9*min(std(yboots),iqr(yboots)/1.34)*length(yboots)^(-0.2);
    
    fx =@(at) ksdensity(xboots,at,'width',h1,'function','pdf');
    fy =@(at) ksdensity(yboots,at,'width',h2,'function','pdf');
    l1(boots)=quadgk(@(x) sqrt(fx(x).^2+fy(x).^2),-Inf,Inf);
    Zboots(boots)=4*sqrt(2)*length(x)*length(y)/(length(x)+length(y)).*(l1(boots)-sqrt(2));

    
    if warn==0
        boots=boots+1;
    end
        
    
end

%======================================================================

Zboots;
if isempty(lastwarn)==1
    exitflag=1;
else
    exitflag=0;
end

subplot(1,2,1)
hist(l1,23)
title('Histogram of the bootsrap distribution of the length values')

subplot(1,2,2)
hist(Zboots,23)
title('Histogram of the bootsrap distribution of the statistic Z')

critical=prctile(Zboots,95);






case 'boxcoxr'  
lastwarn('')

xor=x;yor=y;nx=length(x);ny=length(y);
[transx, transy, lam, exitflag]=boxcoxleo(x,y);
x=transx;
y=transy;

    mx=mean(x);
    sx= std(x);
    my=mean(y);
    sy= std(y);
     
    fx =@(at) normpdf(at,mx,sx);
    fy =@(at) normpdf(at,my,sy);
    len=quadgk(@(x) sqrt(fx(x).^2+fy(x).^2),-Inf,Inf);
    F1inv =@(at) norminv(at,mx,sx);
    F2 =@(at) normcdf(at,my,sy);
    rocfun=@(t) 1-F2(F1inv(1-t));
    
    S=quadgk(@(x) max(0.5.*fx(x),0.5.*fy(x)),-Inf,Inf);
    PLC=(S-0.5)*4/sqrt(2);
    Z=PLC;%4*nx*ny/(nx+ny)*PLC;

    len=quadgk(@(x) sqrt(fx(x).^2+fy(x).^2),-Inf,Inf);
    rocfun=@(t) 1-F2(F1inv(1-t));
    
    Z=4*sqrt(2)*length(x)*length(y)/(length(x)+length(y)).*(len-sqrt(2));


        
  boots=1; 
  pooled=[xor;yor];
%=============START CALCULATING THE LENGTH===========================
while boots<=1000
        rand('twister', 5493+boots);
      randn('state', 3+boots);
      randn('seed',3+boots);
      rand('seed',3+boots);

    warn=0;
    boots;
    if boots==200 || boots==400 || boots==600 || boots==800
        boots;
    end
    
 %ata=randsample(1:nx,nx,'true');
    %atb=randsample(1:ny,ny,'true');
    
    ata=randperm(nx+ny);
    
    pooledboots=pooled(ata);
    xboots=pooledboots(1:nx);
    yboots=pooledboots((nx+1):end);

    
      
    %===KERNEL BASED==========================
    [transx, transy, lam, exitflag]=boxcoxleo(xboots,yboots);
    xboots=transx;
    yboots=transy;
    mxboots=mean(xboots);
    sxboots= std(xboots);
    myboots=mean(yboots);
    syboots= std(yboots);
    
    fx =@(at) normpdf(at,mxboots,sxboots);
    fy =@(at) normpdf(at,myboots,syboots);
    
    l1(boots)=quadgk(@(x) sqrt(fx(x).^2+fy(x).^2),-Inf,Inf);
    Zboots(boots)=4*sqrt(2)*length(x)*length(y)/(length(x)+length(y)).*(l1(boots)-sqrt(2));

    
    if warn==0
        boots=boots+1;
    end
        
    
end

%======================================================================

Zboots;
if isempty(lastwarn)==1
    exitflag=1;
else
    exitflag=0;
end

subplot(1,2,1)
hist(l1,23)
title('Histogram of the bootsrap distribution of the length values')

subplot(1,2,2)
hist(Zboots,23)
title('Histogram of the bootsrap distribution of the statistic Z')

critical=prctile(Zboots,95);






case 'normalr'  
lastwarn('')

xor=x;yor=y;nx=length(x);ny=length(y);
transx=x;
transy=y;

    mx=mean(x);
    sx= std(x);
    my=mean(y);
    sy= std(y);
     
    fx =@(at) normpdf(at,mx,sx);
    fy =@(at) normpdf(at,my,sy);
    len=quadgk(@(x) sqrt(fx(x).^2+fy(x).^2),-Inf,Inf);
    F1inv =@(at) norminv(at,mx,sx);
    F2 =@(at) normcdf(at,my,sy);
    rocfun=@(t) 1-F2(F1inv(1-t));
    

    
    len=quadgk(@(x) sqrt(fx(x).^2+fy(x).^2),-Inf,Inf);
    rocfun=@(t) 1-F2(F1inv(1-t));
    
    Z=4*sqrt(2)*length(x)*length(y)/(length(x)+length(y)).*(len-sqrt(2));


        
  boots=1; 
  pooled=[xor;yor];
%=============START CALCULATING THE LENGTH===========================
while boots<=1000
        rand('twister', 5493+boots);
      randn('state', 3+boots);
      randn('seed',3+boots);
      rand('seed',3+boots);

    warn=0;
    boots;
    if boots==200 || boots==400 || boots==600 || boots==800
        boots;
    end
    
 %ata=randsample(1:nx,nx,'true');
    %atb=randsample(1:ny,ny,'true');
    
    ata=randperm(nx+ny);
    
    pooledboots=pooled(ata);
    xboots=pooledboots(1:nx);
    yboots=pooledboots((nx+1):end);

    
      
    %===KERNEL BASED==========================
    

    mxboots=mean(xboots);
    sxboots= std(xboots);
    myboots=mean(yboots);
    syboots= std(yboots);
    
    fx =@(at) normpdf(at,mxboots,sxboots);
    fy =@(at) normpdf(at,myboots,syboots);
    
    l1(boots)=quadgk(@(x) sqrt(fx(x).^2+fy(x).^2),-Inf,Inf);
    Zboots(boots)=4*sqrt(2)*length(x)*length(y)/(length(x)+length(y)).*(l1(boots)-sqrt(2));

    
    if warn==0
        boots=boots+1;
    end
        
    
end

%======================================================================

Zboots;
if isempty(lastwarn)==1
    exitflag=1;
else
    exitflag=0;
end

subplot(1,2,1)
hist(l1,23)
title('Histogram of the bootsrap distribution of the length values')

subplot(1,2,2)
hist(Zboots,23)
title('Histogram of the bootsrap distribution of the statistic Z')

critical=prctile(Zboots,95);

end







end






